// TSphere2.cpp: implementation of the TSphere class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "TSphere.h"
#include <math.h>
#include "MathLibrary.h"
#include "MemoryManagement.h"
#include "3DGraphicsLibrarySmall.h"


#pragma warning(disable: 4244)	//Shut up about double to sreal casting
#pragma warning(disable: 4305)	//'=' : truncation from 'const double' to 'sreal'










//PURPOSE:	Calculate the texture coordinates for the sphere for a specific texture unit.
//
//PARAMETERS:
//			[in] texUnit : Must be between 0 and 7 inclusively. We need 2 components at minimum.
//			[in] numberOfComponents : 2, 3 or 4
//			[in] isGlobeMapOn  : If TRUE, then texture mapping will be such that we have a 'globe', meaning
//			                     an entire texture will be wrapped around the sphere. This is perfect for games.
//			                     s maps CCW looking from the +z direction, starting from 0.0 and comming back to 1.0 when it
//			                     returns to the same position (but different vertex).
//			                     t maps from +z to -z as 0.0 to 1.0.
//			[in] invertS : If FALSE and if isGlobeMapOn == TRUE, 0.0 to 1.0 CCW, else 1.0 to 0.0
//			[in] invertT : If FALSE and if isGlobeMapOn == TRUE, 0.0 to 1.0 , else 1.0 to 0.0
//			[in] texturingMode : Is one of many modes, look in defines header file. (ignored)
//			[in] chooseVertex : Is the vertices index where we begin texturing. Texture coordinates are (0, 0) (ignored)
//			[in] chooseVertex : Does not depend on (x, y, z) triplet, so it must be multiplied by three.
//			[in] orientationAngle : The sphere has an intrinsic orientation in memory, and this value
//			                        is used to orient the texture at chooseVertex.
//			[in] widthTextureLimit : Is the maximum width of the texture (this is for the future, leave it on 1.0 for now.)
//			[in] heightTextureLimit : Is the maximum width of the texture (this is for the future, leave it on 1.0 for now.)
//			[in] sScale: Is the width ratio to space units. (ignored)
//			[in] tScale: Is the height ratio to space unts. (ignored)
//
//RETURN:
//			-1 if texUnit is out of range.
//			-2 if numberOfComponents is not 2, 3 or 4.
//			-3 no vertices have been generated yet.
//			1 if successful.
//			-10 for array related array (DEBUG) remove it in the future.
//
//NOTE: I have only implemented numberOfComponents==2.
sint TSphere::GenerateTexCoords(sint texUnit, sint numberOfComponents, sint isGlobeMapOn, sint invertS, sint invertT, sint texturingMode, sint chooseVertex, sreal orientationAngle, sreal widthTextureLimit, sreal heightTextureLimit, sreal sScale, sreal tScale)
{
	sint i, j, k, tempo;
	sreal FTempo, FTempo2;
	sreal texIncrementS, texIncrementT;		//Increment factors in texture coordinates

	if((texUnit<0)||(texUnit>=NUMBEROFTEXTUREUNITS))
	{
		strcpy(ErrorMessage, "Tex unit is out of range. @ GenerateTexCoords 496");
		return -1;
	}

	if((numberOfComponents!=2)&&(numberOfComponents!=3)&&(numberOfComponents!=4))
	{
		strcpy(ErrorMessage, "Number of components for texcoord is not 2, 3 or 4. @ GenerateTexCoords 59");
		return -2;
	}

	if(UsedVertex<=0)
	{
		strcpy(ErrorMessage, "Vertex count is zero or less. Can't compute normal. @ GenerateTexCoords 68");
		return -3;
	}

	if(isGlobeMapOn)
	{
		UsedTexCoords[texUnit]=0;
		//DeallocateMemoryForTexCoords(texUnit);
		NumberOfComponentsForTexCoords[texUnit]=numberOfComponents;

		tempo=UsedVertex/NumberOfComponentsForVertex*NumberOfComponentsForTexCoords[texUnit];
		AllocateExact_1(pTexCoords[texUnit], &UsedTexCoords[texUnit], tempo);
		UsedTexCoords[texUnit]=tempo;

		//////////////////////////////////////////////////
		texIncrementS=(sreal)1.0/(sreal)Slices;
		texIncrementT=(sreal)1.0/(sreal)Stacks;

		if(NumberOfComponentsForTexCoords[texUnit]==2)
		{
			//Top of the sphere (triangle_fan)
			//The center-bottom of the texture applied to top of sphere
			i=0;
			if(i+1>=UsedTexCoords[texUnit])
			{
				return -10;		//Boundary error!!!!! (DEBUG)
			}

			pTexCoords[texUnit][i  ]=0.5;
			if(invertT==FALSE)
				pTexCoords[texUnit][i+1]=0.0;
			else
				pTexCoords[texUnit][i+1]=1.0;

			i+=2;
			FTempo=0.0;
			if(invertT==FALSE)
				FTempo2=texIncrementT;
			else
			{
				FTempo2=1.0-texIncrementT;
				texIncrementT=-texIncrementT;
			}

			if(invertS==TRUE)
				texIncrementS=-texIncrementS;

			for(j=1; j<Stacks; j++)
			{
				if(invertS==FALSE)
					FTempo=0.0;
				else
					FTempo=1.0;
				for(k=0; k<Slices; k++, i+=2)
				{
					if(i+1>=UsedTexCoords[texUnit])
					{
						return -10;		//Boundary error!!!!! (DEBUG)
					}
					pTexCoords[texUnit][i  ]=FTempo;
					pTexCoords[texUnit][i+1]=FTempo2;
					FTempo+=texIncrementS;
				}
				if(i+1>=UsedTexCoords[texUnit])
				{
					return -10;		//Boundary error!!!!! (DEBUG)
				}
				//Back to initial position
				if(invertS==FALSE)
					pTexCoords[texUnit][i  ]=1.0;
				else
					pTexCoords[texUnit][i  ]=0.0;
				pTexCoords[texUnit][i+1]=FTempo2;
				i+=2;
				FTempo2+=texIncrementT;
			}
			
			//Bottom of the sphere
			if(i+1>=UsedTexCoords[texUnit])
			{
				return -10;		//Boundary error!!!!! (DEBUG)
			}
			pTexCoords[texUnit][i  ]=0.5;
			if(invertT==FALSE)
				pTexCoords[texUnit][i+1]=1.0;
			else
				pTexCoords[texUnit][i+1]=0.0;
			return 1;
		}
	}
	
	return -1;
}

//PURPOSE:	Be sure to call GenerateNormals() before calling this.
//			WE NEED TO GENERATE BOTH VECTORS AT THE SAME TIME.
//			Indices are required.
//			Vertices are required.
//			Normals are required.
//			Texcoord[0] is required.
//			
//
//RETURNS:
//			1 for success.
//			-1 if UsedIndex<=0
//			-2 if UsedVertex<=0
//			-3 if UsedNormals<=0
//			-4 if UsedTexCoords[0]<=0
//			-5 if bad value passed to ComputeTangentAndBinormalVectors_v1_FLOAT_1 or ComputeTangentAndBinormalVectors_v1_FLOAT_2
sint TSphere::GenerateTangentAndBinormalVectors()
{
	sint tempo, tempo2;

	if(UsedIndex<=0)
	{
		strcpy(ErrorMessage, "Index count is zero or less. @ GenerateTangentAndBinormalVectors 195");
		return -1;
	}

	if(UsedVertex<=0)
	{
		strcpy(ErrorMessage, "Vertex count is zero or less. @ GenerateTangentAndBinormalVectors 201");
		return -2;
	}

	if(UsedNormals<=0)
	{
		strcpy(ErrorMessage, "Normal count is zero or less. @ GenerateTangentAndBinormalVectors 207");
		return -3;
	}

	if(UsedTexCoords[0]<=0)
	{
		strcpy(ErrorMessage, "Texcoord[0] count is zero or less. @ GenerateTangentAndBinormalVectors 213");
		return -4;
	}

	UsedTangents=0;
	UsedBinormals=0;
	//DeallocateMemoryForTangentVectors();
	//DeallocateMemoryForBinormalVectors();

	//One per vertex
	tempo=UsedVertex/NumberOfComponentsForVertex*NumberOfComponentsForTangents;
	AllocateExact_1(pTangents, &UsedTangents, tempo);

	tempo2=UsedVertex/NumberOfComponentsForVertex*NumberOfComponentsForBinormals;
	AllocateExact_1(pBinormals, &UsedBinormals, tempo);

	if(IndexPrecision==INDEXFORMAT_16BIT)
	{
		sint ReturnVal=ComputeTangentAndBinormalVectors_v1_FLOAT_2(UsedIndex, pIndex16Bit, UsedVertex, NumberOfComponentsForVertex, pVertex, NumberOfComponentsForNormals, pNormals, NumberOfComponentsForTexCoords[0],
			pTexCoords[0], NumberOfComponentsForTangents, pTangents, NumberOfComponentsForBinormals, pBinormals);
		if(ReturnVal==-1)
		{
			strcpy(ErrorMessage, "Bad value passed.  @ GenerateTangentAndBinormalVectors 243");
			return -5;
		}
		else if(ReturnVal==0)
		{
			strcpy(ErrorMessage, "Memory error detected @ GenerateTangentAndBinormalVectors 248");
			return 0;
		}
	}
	else if(IndexPrecision==INDEXFORMAT_32BIT)
	{
		sint ReturnVal=ComputeTangentAndBinormalVectors_v1_FLOAT(UsedIndex, pIndex32Bit, UsedVertex, NumberOfComponentsForVertex, pVertex, NumberOfComponentsForNormals, pNormals, NumberOfComponentsForTexCoords[0],
			pTexCoords[0], NumberOfComponentsForTangents, pTangents, NumberOfComponentsForBinormals, pBinormals);
		if(ReturnVal==-1)
		{
			strcpy(ErrorMessage, "Bad value passed.  @ GenerateTangentAndBinormalVectors 258");
			return -5;
		}
		else if(ReturnVal==0)
		{
			strcpy(ErrorMessage, "Memory error detected @ GenerateTangentAndBinormalVectors 263");
			return 0;
		}
	}

	UsedTangents=tempo;
	UsedBinormals=tempo2;

	return 1;
}

//PURPOSE:	As powerful as StartupByABCStacksAndSlices, but we added startingStack and endingStack.
//			startingStack is the first stack and endingStack is the last one to be included.
//			Example : If stacks = 10 and 
//			startingStack = 0
//			endingStack = 2
//
//			we will generate stack 1, 2 and 3
//
//PARAMETERS:
//			[in] centerXYZ : XYZ position of center. Can be NULL. Default is {0, 0, 0}(Not used for now)
//			[in] orientation : XYZ vector, need not be normalized. Can be NULL. Default is {0, 0, 1} (Not used for now)
//			[in] radius : >0.0, else set to 1.0
//			[in] stacks : the number of layer about the z-axis. >=3
//			[in] slices : the number of slices about the z-axis. >=3
//			[in] isGenerateIndex : indices that could be sent to GL.
//			[in] startingStack : must be 0 to stacks-1, the first value (stacks must be valid)
//			[in] endingStack : must be 0 to stacks-1, the second value (stacks must be valid)
//
//RETURN:
//			1 for success
//			0 for memory error.
//			-1 for array related array (DEBUG) remove it in the future.
sint TSphere::StartupByABCStacksAndSlices_StackBound(sreal *centerXYZ, sreal *orientation, sreal aRadius, sreal bRadius, sreal cRadius, sint stacks, sint slices, tbool isGenerateIndex,
		sint startingStack, sint endingStack)
{
	Slices=(slices<3) ? 3 : slices;
	Stacks=(stacks<3) ? 3 : stacks;

	if(startingStack<0)
		StartingStack=0;
	else if(startingStack>=Stacks)
		StartingStack=Stacks-1;
	else
		StartingStack=startingStack;

	if(endingStack>=Stacks)
		EndingStack=Stacks-1;
	else if(endingStack<StartingStack)
		EndingStack=StartingStack;
	else
		EndingStack=endingStack;

	PolygonNumber=2*Slices+(Stacks-2)*Slices*2;		//Triangle count
	
	RadiusA=(aRadius<=0.0) ? 1.0 : aRadius;
	RadiusB=(bRadius<=0.0) ? 1.0 : bRadius;
	RadiusC=(cRadius<=0.0) ? 1.0 : cRadius;

	return SameStartupPatternForSphere(centerXYZ, orientation, isGenerateIndex);
}


//PURPOSE:	Calculate all the vertices.
//			There won't be any duplicate vertices. Indices must be generated as well.
//
//RETURN:
//			1 for success.
//			-1 for array related array (DEBUG) remove it in the future.
sint TSphere::GenerateVertices2()
{
	sint i, j, k, m, ReturnVal;
	sreal FTempo3, FTempo4;

	//The top and bottom of the sphere is made of triangle_fan.
	//The rest is made with triangle strips.
	//Indices will of course be GL_TRIANGLES.

	DeallocateMemoryForIndex();
	DeallocateMemoryForVertex();
	DeallocateMemoryForNormals();
	for(i=0; i<NUMBEROFTEXTUREUNITS; i++)
	{
		DeallocateMemoryForTexCoords(i);
	}
	DeallocateMemoryForColors();
	DeallocateMemoryForSecondaryColors();
	DeallocateMemoryForTangentVectors();
	DeallocateMemoryForBinormalVectors();

	DeallocateMemoryVertexInterlaced();

	VertexCount=1+1+(Stacks-1)*(Slices+1);			//One for the top, one for the bottom, the rest is in the body
	Start_DrawRangeElements=0;
	End_DrawRangeElements=VertexCount-1;


	//For tangent and binormal generation, indices are needed so generate them first
	if(IsGenerateIndex)
	{
		GenerateIndices();
	}

	if(VertexFormat==GLHVERTEXFORMAT_V)
	{
		if((StartingStack==0)&&(EndingStack==Stacks-1))		//Generate the full sphere
		{
			pVertex_V=new GLHVertex_V[VertexCount];


			sreal PhiIncrement=(2.0*PI_DEF)/(sreal)Slices, Phi;
			sreal ThetaIncrement=PI_DEF/(sreal)Stacks, Theta;

			//Top vertex
			pVertex_V[0].x=0.0;
			pVertex_V[0].y=0.0;
			pVertex_V[0].z=RadiusC;

			Theta=ThetaIncrement;
			Phi=0.0;
			FTempo3=sinf(Theta);
			FTempo4=RadiusC*cosf(Theta);

			i=1;
			for(j=1; j<Stacks; j++)
			{
				Phi=0.0;
				m=i;									//Save start index
				for(k=0; k<Slices; k++, i++)
				{
					if(i>=VertexCount)
					{
						return -1;		//Boundary error!!!!! (DEBUG)
					}

					pVertex_V[i].x=RadiusA*cosf(Phi)*FTempo3;
					pVertex_V[i].y=RadiusB*sinf(Phi)*FTempo3;
					pVertex_V[i].z=FTempo4;
					Phi+=PhiIncrement;
				}

				if(i>=VertexCount)
				{
					return -1;		//Boundary error!!!!! (DEBUG)
				}

				pVertex_V[i].x=pVertex_V[m].x;
				pVertex_V[i].y=pVertex_V[m].y;
				pVertex_V[i].z=pVertex_V[m].z;
				i++;
				Theta+=ThetaIncrement;
				FTempo3=sinf(Theta);
				FTempo4=RadiusC*cosf(Theta);
			}

			//Bottom vertex
			if(i>=VertexCount)
			{
				return -1;		//Boundary error!!!!! (DEBUG)
			}

			pVertex_V[i].x=0.0;
			pVertex_V[i].y=0.0;
			pVertex_V[i].z=-RadiusC;
		}
	}
	else if(VertexFormat==GLHVERTEXFORMAT_VT)
	{
		if((StartingStack==0)&&(EndingStack==Stacks-1))		//Generate the full sphere
		{
			pVertex_VT=new GLHVertex_VT[VertexCount];


			sreal PhiIncrement=(2.0*PI_DEF)/(sreal)Slices, Phi;
			sreal ThetaIncrement=PI_DEF/(sreal)Stacks, Theta;

			//Top vertex
			pVertex_VT[0].x=0.0;
			pVertex_VT[0].y=0.0;
			pVertex_VT[0].z=RadiusC;

			Theta=ThetaIncrement;
			Phi=0.0;
			FTempo3=sinf(Theta);
			FTempo4=RadiusC*cosf(Theta);

			i=1;
			for(j=1; j<Stacks; j++)
			{
				Phi=0.0;
				m=i;									//Save start index
				for(k=0; k<Slices; k++, i++)
				{
					if(i>=VertexCount)
					{
						return -1;		//Boundary error!!!!! (DEBUG)
					}

					pVertex_VT[i].x=RadiusA*cosf(Phi)*FTempo3;
					pVertex_VT[i].y=RadiusB*sinf(Phi)*FTempo3;
					pVertex_VT[i].z=FTempo4;
					Phi+=PhiIncrement;
				}

				if(i>=VertexCount)
				{
					return -1;		//Boundary error!!!!! (DEBUG)
				}

				pVertex_VT[i].x=pVertex_VT[m].x;
				pVertex_VT[i].y=pVertex_VT[m].y;
				pVertex_VT[i].z=pVertex_VT[m].z;
				i++;
				Theta+=ThetaIncrement;
				FTempo3=sinf(Theta);
				FTempo4=RadiusC*cosf(Theta);
			}

			//Bottom vertex
			if(i>=VertexCount)
			{
				return -1;		//Boundary error!!!!! (DEBUG)
			}

			pVertex_VT[i].x=0.0;
			pVertex_VT[i].y=0.0;
			pVertex_VT[i].z=-RadiusC;
		}
	}
	else if(VertexFormat==GLHVERTEXFORMAT_VNT)
	{
		if((StartingStack==0)&&(EndingStack==Stacks-1))		//Generate the full sphere
		{
			pVertex_VNT=new GLHVertex_VNT[VertexCount];


			sreal PhiIncrement=(2.0*PI_DEF)/(sreal)Slices, Phi;
			sreal ThetaIncrement=PI_DEF/(sreal)Stacks, Theta;

			//Top vertex
			pVertex_VNT[0].x=0.0;
			pVertex_VNT[0].y=0.0;
			pVertex_VNT[0].z=RadiusC;

			Theta=ThetaIncrement;
			Phi=0.0;
			FTempo3=sinf(Theta);
			FTempo4=RadiusC*cosf(Theta);

			i=1;
			for(j=1; j<Stacks; j++)
			{
				Phi=0.0;
				m=i;									//Save start index
				for(k=0; k<Slices; k++, i++)
				{
					if(i>=VertexCount)
					{
						return -1;		//Boundary error!!!!! (DEBUG)
					}

					pVertex_VNT[i].x=RadiusA*cosf(Phi)*FTempo3;
					pVertex_VNT[i].y=RadiusB*sinf(Phi)*FTempo3;
					pVertex_VNT[i].z=FTempo4;
					Phi+=PhiIncrement;
				}

				if(i>=VertexCount)
				{
					return -1;		//Boundary error!!!!! (DEBUG)
				}

				pVertex_VNT[i].x=pVertex_VNT[m].x;
				pVertex_VNT[i].y=pVertex_VNT[m].y;
				pVertex_VNT[i].z=pVertex_VNT[m].z;
				i++;
				Theta+=ThetaIncrement;
				FTempo3=sinf(Theta);
				FTempo4=RadiusC*cosf(Theta);
			}

			//Bottom vertex
			if(i>=VertexCount)
			{
				return -1;		//Boundary error!!!!! (DEBUG)
			}

			pVertex_VNT[i].x=0.0;
			pVertex_VNT[i].y=0.0;
			pVertex_VNT[i].z=-RadiusC;
		}

		///////////////////////////////////////
		//Normals
		//The sphere is generate with center at {0, 0, 0}
		for(i=0; i<VertexCount; i++)
		{
			pVertex_VNT[i].nx=pVertex_VNT[i].x;
			pVertex_VNT[i].ny=pVertex_VNT[i].y;
			pVertex_VNT[i].nz=pVertex_VNT[i].z;
			NormalizeVectorFLOAT_2(&pVertex_VNT[i].nx);
		}

		GenerateTexCoords2(FALSE, FALSE);
	}
	else if(VertexFormat==GLHVERTEXFORMAT_VNT3)
	{
		if((StartingStack==0)&&(EndingStack==Stacks-1))		//Generate the full sphere
		{
			pVertex_VNT3=new GLHVertex_VNT3[VertexCount];
			//To set the r component to 0.0
			memset(pVertex_VNT3, 0, sizeof(GLHVertex_VNT3)*VertexCount);

			sreal PhiIncrement=(2.0*PI_DEF)/(sreal)Slices, Phi;
			sreal ThetaIncrement=PI_DEF/(sreal)Stacks, Theta;

			//Top vertex
			pVertex_VNT3[0].x=0.0;
			pVertex_VNT3[0].y=0.0;
			pVertex_VNT3[0].z=RadiusC;

			Theta=ThetaIncrement;
			Phi=0.0;
			FTempo3=sinf(Theta);
			FTempo4=RadiusC*cosf(Theta);

			i=1;
			for(j=1; j<Stacks; j++)
			{
				Phi=0.0;
				m=i;									//Save start index
				for(k=0; k<Slices; k++, i++)
				{
					if(i>=VertexCount)
					{
						return -1;		//Boundary error!!!!! (DEBUG)
					}

					pVertex_VNT3[i].x=RadiusA*cosf(Phi)*FTempo3;
					pVertex_VNT3[i].y=RadiusB*sinf(Phi)*FTempo3;
					pVertex_VNT3[i].z=FTempo4;
					Phi+=PhiIncrement;
				}

				if(i>=VertexCount)
				{
					return -1;		//Boundary error!!!!! (DEBUG)
				}

				pVertex_VNT3[i].x=pVertex_VNT3[m].x;
				pVertex_VNT3[i].y=pVertex_VNT3[m].y;
				pVertex_VNT3[i].z=pVertex_VNT3[m].z;
				i++;
				Theta+=ThetaIncrement;
				FTempo3=sinf(Theta);
				FTempo4=RadiusC*cosf(Theta);
			}

			//Bottom vertex
			if(i>=VertexCount)
			{
				return -1;		//Boundary error!!!!! (DEBUG)
			}

			pVertex_VNT3[i].x=0.0;
			pVertex_VNT3[i].y=0.0;
			pVertex_VNT3[i].z=-RadiusC;
		}

		///////////////////////////////////////
		//Normals
		//The sphere is generate with center at {0, 0, 0}
		for(i=0; i<VertexCount; i++)
		{
			pVertex_VNT3[i].nx=pVertex_VNT3[i].x;
			pVertex_VNT3[i].ny=pVertex_VNT3[i].y;
			pVertex_VNT3[i].nz=pVertex_VNT3[i].z;
			NormalizeVectorFLOAT_2(&pVertex_VNT3[i].nx);
		}

		GenerateTexCoords2(FALSE, FALSE);
	}
	else if(VertexFormat==GLHVERTEXFORMAT_VNTT3T3)
	{
		if((StartingStack==0)&&(EndingStack==Stacks-1))		//Generate the full sphere
		{
			pVertex_VNTT3T3=new GLHVertex_VNTT3T3[VertexCount];
			//To set the r component to 0.0
			memset(pVertex_VNTT3T3, 0, sizeof(GLHVertex_VNTT3T3)*VertexCount);

			sreal PhiIncrement=(2.0*PI_DEF)/(sreal)Slices, Phi;
			sreal ThetaIncrement=PI_DEF/(sreal)Stacks, Theta;

			//Top vertex
			pVertex_VNTT3T3[0].x=0.0;
			pVertex_VNTT3T3[0].y=0.0;
			pVertex_VNTT3T3[0].z=RadiusC;

			Theta=ThetaIncrement;
			Phi=0.0;
			FTempo3=sinf(Theta);
			FTempo4=RadiusC*cosf(Theta);

			i=1;
			for(j=1; j<Stacks; j++)
			{
				Phi=0.0;
				m=i;									//Save start index
				for(k=0; k<Slices; k++, i++)
				{
					if(i>=VertexCount)
					{
						return -1;		//Boundary error!!!!! (DEBUG)
					}

					pVertex_VNTT3T3[i].x=RadiusA*cosf(Phi)*FTempo3;
					pVertex_VNTT3T3[i].y=RadiusB*sinf(Phi)*FTempo3;
					pVertex_VNTT3T3[i].z=FTempo4;
					Phi+=PhiIncrement;
				}

				if(i>=VertexCount)
				{
					return -1;		//Boundary error!!!!! (DEBUG)
				}

				pVertex_VNTT3T3[i].x=pVertex_VNTT3T3[m].x;
				pVertex_VNTT3T3[i].y=pVertex_VNTT3T3[m].y;
				pVertex_VNTT3T3[i].z=pVertex_VNTT3T3[m].z;
				i++;
				Theta+=ThetaIncrement;
				FTempo3=sinf(Theta);
				FTempo4=RadiusC*cosf(Theta);
			}

			//Bottom vertex
			if(i>=VertexCount)
			{
				return -1;		//Boundary error!!!!! (DEBUG)
			}

			pVertex_VNTT3T3[i].x=0.0;
			pVertex_VNTT3T3[i].y=0.0;
			pVertex_VNTT3T3[i].z=-RadiusC;
		}

		///////////////////////////////////////
		//Normals
		//The sphere is generate with center at {0, 0, 0}
		for(i=0; i<VertexCount; i++)
		{
			pVertex_VNTT3T3[i].nx=pVertex_VNTT3T3[i].x;
			pVertex_VNTT3T3[i].ny=pVertex_VNTT3T3[i].y;
			pVertex_VNTT3T3[i].nz=pVertex_VNTT3T3[i].z;
			NormalizeVectorFLOAT_2(&pVertex_VNTT3T3[i].nx);
		}

		GenerateTexCoords2(FALSE, FALSE);

		if(IndexPrecision==INDEXFORMAT_16BIT)
		{
			ReturnVal=ComputeTangentAndBinormalVectors_v1_FLOAT_4(UsedIndex, pIndex16Bit, VertexCount, pVertex_VNTT3T3);
			if(ReturnVal==-1)
			{
				strcpy(ErrorMessage, "Bad value passed.  @ GenerateTangentAndBinormalVectors 712");
				return -1;
			}
		}
		else if(IndexPrecision==INDEXFORMAT_32BIT)
		{
			ReturnVal=ComputeTangentAndBinormalVectors_v1_FLOAT_3(UsedIndex, pIndex32Bit, VertexCount, pVertex_VNTT3T3);
			if(ReturnVal==-1)
			{
				strcpy(ErrorMessage, "Bad value passed.  @ GenerateTangentAndBinormalVectors 721");
				return -1;
			}
		}
	}

	return 1;
}

//PURPOSE:	Calculate the texture coordinates for the sphere for a specific texture unit.
//
//PARAMETERS:
//			[in] invertS : If FALSE and if isGlobeMapOn == TRUE, 0.0 to 1.0 CCW, else 1.0 to 0.0
//			[in] invertT : If FALSE and if isGlobeMapOn == TRUE, 0.0 to 1.0 , else 1.0 to 0.0
//
//RETURN:
//			1 if successful
//			-1 if isGlobeMapOn==FALSE
sint TSphere::GenerateTexCoords2(sint invertS, sint invertT)
{
	sint i, j, k;
	sreal FTempo, FTempo2;
	sreal texIncrementS, texIncrementT;		//Increment factors in texture coordinates

	if(TexCoordStyle==1)
	{
		//////////////////////////////////////////////////
		texIncrementS=(sreal)1.0/(sreal)Slices;
		texIncrementT=(sreal)1.0/(sreal)Stacks;

		if(VertexFormat==GLHVERTEXFORMAT_VT)
		{
			//Top of the sphere (triangle_fan)
			//The center-bottom of the texture applied to top of sphere
			i=0;
			pVertex_VT[i].s=0.5;
			if(invertT==FALSE)
				pVertex_VT[i].t=0.0;
			else
				pVertex_VT[i].t=1.0;

			i++;
			FTempo=0.0;
			if(invertT==FALSE)
				FTempo2=texIncrementT;
			else
			{
				FTempo2=1.0-texIncrementT;
				texIncrementT=-texIncrementT;
			}

			if(invertS==TRUE)
				texIncrementS=-texIncrementS;

			for(j=1; j<Stacks; j++)
			{
				if(invertS==FALSE)
					FTempo=0.0;
				else
					FTempo=1.0;

				for(k=0; k<Slices; k++, i++)
				{
					pVertex_VT[i].s=FTempo;
					pVertex_VT[i].t=FTempo2;
					FTempo+=texIncrementS;
				}

				//Back to initial position
				if(invertS==FALSE)
					pVertex_VT[i].s=1.0;
				else
					pVertex_VT[i].s=0.0;

				pVertex_VT[i].t=FTempo2;
				i++;
				FTempo2+=texIncrementT;
			}

			pVertex_VT[i].s=0.5;
			if(invertT==FALSE)
				pVertex_VT[i].t=1.0;
			else
				pVertex_VT[i].t=0.0;

			return 1;
		}
		else if(VertexFormat==GLHVERTEXFORMAT_VNT)
		{
			//Top of the sphere (triangle_fan)
			//The center-bottom of the texture applied to top of sphere
			i=0;
			pVertex_VNT[i].s=0.5;
			if(invertT==FALSE)
				pVertex_VNT[i].t=0.0;
			else
				pVertex_VNT[i].t=1.0;

			i++;
			FTempo=0.0;
			if(invertT==FALSE)
				FTempo2=texIncrementT;
			else
			{
				FTempo2=1.0-texIncrementT;
				texIncrementT=-texIncrementT;
			}

			if(invertS==TRUE)
				texIncrementS=-texIncrementS;

			for(j=1; j<Stacks; j++)
			{
				if(invertS==FALSE)
					FTempo=0.0;
				else
					FTempo=1.0;

				for(k=0; k<Slices; k++, i++)
				{
					pVertex_VNT[i].s=FTempo;
					pVertex_VNT[i].t=FTempo2;
					FTempo+=texIncrementS;
				}

				//Back to initial position
				if(invertS==FALSE)
					pVertex_VNT[i].s=1.0;
				else
					pVertex_VNT[i].s=0.0;

				pVertex_VNT[i].t=FTempo2;
				i++;
				FTempo2+=texIncrementT;
			}

			pVertex_VNT[i].s=0.5;
			if(invertT==FALSE)
				pVertex_VNT[i].t=1.0;
			else
				pVertex_VNT[i].t=0.0;

			return 1;
		}
		else if(VertexFormat==GLHVERTEXFORMAT_VNT3)
		{
			//Top of the sphere (triangle_fan)
			//The center-bottom of the texture applied to top of sphere
			i=0;
			pVertex_VNT3[i].s=0.5;
			if(invertT==FALSE)
				pVertex_VNT3[i].t=0.0;
			else
				pVertex_VNT3[i].t=1.0;

			i++;
			FTempo=0.0;
			if(invertT==FALSE)
				FTempo2=texIncrementT;
			else
			{
				FTempo2=1.0-texIncrementT;
				texIncrementT=-texIncrementT;
			}

			if(invertS==TRUE)
				texIncrementS=-texIncrementS;

			for(j=1; j<Stacks; j++)
			{
				if(invertS==FALSE)
					FTempo=0.0;
				else
					FTempo=1.0;

				for(k=0; k<Slices; k++, i++)
				{
					pVertex_VNT3[i].s=FTempo;
					pVertex_VNT3[i].t=FTempo2;
					FTempo+=texIncrementS;
				}

				//Back to initial position
				if(invertS==FALSE)
					pVertex_VNT3[i].s=1.0;
				else
					pVertex_VNT3[i].s=0.0;

				pVertex_VNT3[i].t=FTempo2;
				i++;
				FTempo2+=texIncrementT;
			}

			pVertex_VNT3[i].s=0.5;
			if(invertT==FALSE)
				pVertex_VNT3[i].t=1.0;
			else
				pVertex_VNT3[i].t=0.0;

			return 1;
		}
		else if(VertexFormat==GLHVERTEXFORMAT_VNTT3T3)
		{
			//Top of the sphere (triangle_fan)
			//The center-bottom of the texture applied to top of sphere
			i=0;
			pVertex_VNTT3T3[i].s=0.5;
			if(invertT==FALSE)
				pVertex_VNTT3T3[i].t=0.0;
			else
				pVertex_VNTT3T3[i].t=1.0;

			i++;
			FTempo=0.0;
			if(invertT==FALSE)
				FTempo2=texIncrementT;
			else
			{
				FTempo2=1.0-texIncrementT;
				texIncrementT=-texIncrementT;
			}

			if(invertS==TRUE)
				texIncrementS=-texIncrementS;

			for(j=1; j<Stacks; j++)
			{
				if(invertS==FALSE)
					FTempo=0.0;
				else
					FTempo=1.0;

				for(k=0; k<Slices; k++, i++)
				{
					pVertex_VNTT3T3[i].s=FTempo;
					pVertex_VNTT3T3[i].t=FTempo2;
					FTempo+=texIncrementS;
				}

				//Back to initial position
				if(invertS==FALSE)
					pVertex_VNTT3T3[i].s=1.0;
				else
					pVertex_VNTT3T3[i].s=0.0;

				pVertex_VNTT3T3[i].t=FTempo2;
				i++;
				FTempo2+=texIncrementT;
			}

			pVertex_VNTT3T3[i].s=0.5;
			if(invertT==FALSE)
				pVertex_VNTT3T3[i].t=1.0;
			else
				pVertex_VNTT3T3[i].t=0.0;

			return 1;
		}
	}

	return -1;
}

//PURPOSE:
//
//PARAMETERS:
//
//RETURN:
//


